package binomialHeap.concurrent

// import ox.cads.util.ThreadID
import java.util.concurrent.atomic.AtomicReference

/** An indirect reference to a heap. */
class HeapIndirection(heap: ConcHeap){

  /** A step along the path to the heap corresponding to this: either Left(h)
    * where h is that heap, or Right(hi) where hi is another
    * HeapIndirection. 
    * I think it's necessary to use an AtomicReference to avoid race 
    * conditions between setNext and path compression. */
  // @volatile private var next: Either[ConcHeap, HeapIndirection] = Left(heap)
  private val next =
    new AtomicReference[Either[ConcHeap, HeapIndirection]](Left(heap))

  /** Set this to point to hi.  setNext may be called multiple times, but only
    * the first will succeed. */
  def setNext(hi: HeapIndirection) = {
    val myNext = next.get
    myNext match{
      case Left(h) => next.compareAndSet(myNext, Right(hi))
      case Right(hi1) => {} // already done.
      // The following assertion won't hold with path compression.
      // assert(hi1 == hi, ThreadID.get+": expected "+hi+"; found "+hi1)
      // println(ThreadID.get+" heapIndirection updated consistently")
    }
  }

  /** Get the heap that this indirectly points to (without path compression). */
  def getHeap: ConcHeap = next.get match{
    case Left(h) =>
      val h1 = h.heapOf
      if(next.get == Left(h)) h1 else getHeap
    case Right(hi) => hi.getHeap
  }

  /** Perform path compression, updating next to point to the final
    * HeapIndirection on the path, and returning that HeapIndirection. */
  def compress: HeapIndirection = if(false) this else {
    val myNext = next.get
    myNext match{
      case Left(_) => this
      case Right(hi) =>
        val hi1 = hi.compress
        if(hi != hi1){
          /* heap.log("compressing"); */
          next.compareAndSet(myNext, Right(hi1))
        }
        hi1
    }
  }

  val id = scala.util.Random.nextInt(1000)

  override def toString = {
    val argString = next.get match{
      case Left(h) => "Left("+id+", "+h.id+")"
      case Right(hi) => "Right("+id+", "+hi.toString+")"
    }
    "HeapIndirection("+argString+")"
  }


}
