package binomialHeap.concurrent

import java.util.concurrent.atomic.AtomicReference

/** A label to be stored within a HeapLabelList, indicating an operation that
  * cannot be done concurrently with this heap being the giving heap in a
  * union. */
class HeapLabel{
  /** The next label in the list. */
  val next = new AtomicReference[HeapLabel](null)

  /** Has this operation been completed (perhaps unsuccessfully)? */
  @volatile var done = false
}

// ============

/** Label indicating that a thread is trying to insert node below parent. */
case class InsertBelow(
  node: Node, parent: Node, parentState: NodeState, epoch: Long)
    extends HeapLabel{
  override def toString =
    "InsertBelow"+(node.key, parent.key, epoch)+(if(done) "done" else "")
}

// ============

/** Label findicating that a thread is trying to insert node after pred. */
case class InsertAtEnd(
  node: Node, pred: Node, predState: NodeState, epoch: Long)
    extends HeapLabel{
  override def toString =
    "InsertAfter"+(node.key, pred.key)+(if(done) "done" else "")

  // Has the insertion happened successfully?
  @volatile var success = false
}

// ============

/** Label for the head of the heap indicating that a thread is trying to label
  * delNode, with state delState, for deletion using label delLabel. */
case class LabelForDelete(
  delNode: Node, delState: NodeState, newNext: Node,
  startEpoch: Long, delLabel: Delete)
    extends HeapLabel{
  override def toString =
    "LabelForDelete"+(delNode.key, startEpoch).toString+
      (if(done) "done" else "")
}

// ============

/** This heap is acting as the receiver in a union. */
class UnionReceiver(val giver: Union){
  // Has the union been completed? 
  @volatile var done = false

  override def toString = "UnionReceiver("+giver.id+")"
}
// Note the above doesn't extend HeapLabel
 
 
