/** Program to run the test
  * scala binomialHeap.tests.PQueueTimer --profile --DES --init 1000 
  *   --maxKey 1000 --A 8 --B 8 -p <p> --iters <iters> --maxTime <maxTime>
  * <reps> times, each run as a different process.  The paramters <iters>,
  * <maxTime>, <p> and <reps> are settable on the command line.
  */

package binomialHeap.tests

import java.io._

object PQueueProfiler{
  // Default values for iters, reps, maxTime
  var iters = 200000
  var reps = 100
  var maxTime = 35000
  var p = 63
  var doDES = false
  // The command to run
  val cmd0 =
    "scala binomialHeap.tests.PQueueTimer --profile --A 8 --B 8 --reps 1"
  def cmd1 =
    cmd0+(
      if(doDES) " --DES --init 1000 --maxKey 1000"
      else " --insertProb 0.4 --minProb 0.2 --delMinProb 0.4 --init 1000"
    )
  def cmd = cmd1+" -p "+p+" --iters "+iters+" --maxTime "+maxTime+
    " --numMinsDelMin 32 --numMinsMin 2 --tidyRatio 16"

  private val rt = java.lang.Runtime.getRuntime()

  /** Run cmd as its own process, echoing output. */
  def runProc(cmd: String) = {
    val proc = rt.exec(cmd)
    val isr = new InputStreamReader(proc.getInputStream, "ISO-8859-1")
    val fromProc = new BufferedReader(isr)
    val isrErr = new InputStreamReader(proc.getErrorStream, "ISO-8859-1")
    val errProc = new BufferedReader(isrErr)
    var tstr = fromProc.readLine
    while(tstr != null){
      println(tstr)
      tstr = fromProc.readLine
    }
    // Flush error stream
    var err = errProc.readLine
    if(err != null){
      do{ println(err); err = errProc.readLine } while(err != null);
      assert(false)
    }
    proc.waitFor
  }

  def main(args: Array[String]) = {
    // Parse arguments
    var i = 0
    while(i < args.length) args(i) match{
      case "--DES" => doDES = true; i += 1
      case "--maxTime" => maxTime = args(i+1).toInt; i += 2
      case "--iters" => iters = args(i+1).toInt; i += 2 
      case "--reps" =>  reps = args(i+1).toInt; i += 2
      case "-p" =>  p = args(i+1).toInt; i += 2
      case _ => println("Unrecognised argument: "+args(i)); sys.exit
    }

    // Run tests
    println(cmd)
    for(i <- 0 until reps){
      if(i%10 == 0) runProc(cmd+" --verbose") else runProc(cmd)
    }
  }

}
