package binomialHeap.tests

/** A sequential specification object, corresponding to a heap containing
  * contents. 
  * Pre: contents is sorted. */
case class SeqMultiSet(contents: List[Int] = List[Int]()){
 
  def insert(key: Int) : (Unit, SeqMultiSet) = {
    val newContents =
      contents.filter(_ < key) ++ (key :: contents.filter (_ >= key))
    ((), new SeqMultiSet(newContents))
  }

  def minimum : (Option[Int], SeqMultiSet) =
    if(contents.isEmpty) (None, this)
    else (Some(contents.head), this)

  def deleteMin() : (Option[Int], SeqMultiSet) =
    if(contents.isEmpty) (None, this)
    else (Some(contents.head), new SeqMultiSet(contents.tail))

  private def merge(list1: List[Int], list2: List[Int]) : List[Int] =
    if(list1.isEmpty) list2
    else if(list2.isEmpty) list1
    else{
      val x1 = list1.head; val x2 = list2.head
      if(x1 <= x2) x1 :: merge(list1.tail, list2)
      else x2 :: merge(list1, list2.tail)
    }

  def union(that: SeqMultiSet) : (Unit, SeqMultiSet) = 
    ((), new SeqMultiSet(merge(contents, that.contents)))
}

// ==================================================================



/** A sequential specification object for a collection of heaps, including the
  * union function.
  * @param contents array containing the heaps. */
class SeqMultiHeap(contents: Array[SeqMultiSet]){
  /** Insert key into heap i. */
  def insert(i: Int, key: Int) : (Unit, SeqMultiHeap) = {
    val newContents = contents.clone
    val ((), newHeap) = contents(i).insert(key)
    newContents(i) = newHeap
    ((), new SeqMultiHeap(newContents))
  }

  /** Minimum key in heap i. */
  def minimum(i: Int) : (Option[Int], SeqMultiHeap) = {
    val (result, _) = contents(i).minimum; (result, this)
  }

  /** Delete minimum element from heap i. */
  def deleteMin(i: Int) : (Option[Int], SeqMultiHeap) = {
    val newContents = contents.clone
    val (result, newHeap) = contents(i).deleteMin()
    newContents(i) = newHeap
    (result, new SeqMultiHeap(newContents))
  }

  /** Add heap j to heap i, and clear heap j. */
  def union(i: Int, j: Int) : (Unit, SeqMultiHeap) = {
    assert(i != j)
    val newContents = contents.clone
    val ((), newHeap) = contents(i).union(contents(j))
    newContents(i) = newHeap; newContents(j) = new SeqMultiSet
    ((), new SeqMultiHeap(newContents))
  }
}

/** Companion object for MultiHeap. */
object SeqMultiHeap{
  /** Factory method to create MultiHeap containing N heaps. */
  def apply(N: Int) = new SeqMultiHeap(Array.fill(N)(SeqMultiSet()))
}
